local AceAddon = AceLibrary("AceAddon-2.0")
local Tablet = AceLibrary("Tablet-2.0")

local L = AceLibrary("AceLocale-2.2"):new("Cartographer_Vendors")
L:RegisterTranslations("enUS", function() return {
	["Filter"] = true,
	["Filter out vendors"] = true,
	
	["Select all"] = true,
	["Select none"] = true,
    
    ["Vendor"] = true,
} end)

L:RegisterTranslations("koKR", function() return {
	["Filter"] = "필터",
	["Filter out vendors"] = "상인 아웃 필터",
	
	["Select all"] = "모두 선택",
	["Select none"] = "선택 없음",

        ["Vendor"] = "상인",
} end)

local mod
if AceLibrary:HasInstance("AceDebug-2.0") then 
    mod = Cartographer:NewModule("Vendors", "AceConsole-2.0", "AceEvent-2.0", "AceDebug-2.0")
    mod:SetDebugging(false)
else
    mod = Cartographer:NewModule("Vendors", "AceConsole-2.0", "AceEvent-2.0")
    -- mod.Debug = function() return end
end

mod.icon = {
	["Vendor"] = {
		text = L["Vendor"],
		path = "Interface\\Icons\\INV_Misc_ElvenCoins",
		width = 12,
		height = 12
	},
}

mod.vendors = {}

function mod:OnInitialize()
	self.db = Cartographer:AcquireDBNamespace("Vendors")
	Cartographer:RegisterDefaults("Vendors", "profile", {
		filter = {
			['*'] = true,
		}
	})

	local aceopts = {}
	aceopts.toggle = {
		name = AceLibrary("AceLocale-2.2"):new("Cartographer")["Enabled"],
		desc = AceLibrary("AceLocale-2.2"):new("Cartographer")["Suspend/resume this module."],
		type  = 'toggle',
		order = -1,
		get   = function() return Cartographer:IsModuleActive(self) end,
		set   = function() Cartographer:ToggleModuleActive(self) end,
	}

    --[[
	aceopts.filter = {
		name = L["Filter"],
		desc = L["Filter out quests"],
		type = 'group',
		args = {
			all = {
				name = L["Select all"],
				desc = L["Select all"],
				type = 'execute',
				func = function()
					for k in pairs(self.icon) do
						self:ToggleShowingQuests(k, true)
					end
				end,
				order = 1,
			},
			none = {
				name = L["Select none"],
				desc = L["Select none"],
				type = 'execute',
				func = function()
					for k in pairs(self.icon) do
						self:ToggleShowingQuests(k, false)
					end
				end,
				order = 2,
			},
			blank = {
				type = 'header',
				order = 3,
			}
		}
	}
	for k,v in pairs(self.icon) do
		local k = k
		aceopts.filter.args[k] = {
			name = v.text,
			desc = v.text,
			type = 'toggle',
			get = function()
				return self:IsShowingQuests(k)
			end,
			set = function(value)
				return self:ToggleShowingQuests(k, value)
			end,
		}
	end
    --]]
    
	Cartographer.options.args["Vendors"] = {
		name = "Vendors",
		desc = self.notes,
		type = 'group',
		args = aceopts,
		handler = self,
	}
	AceLibrary("AceConsole-2.0"):InjectAceOptionsTable(self, Cartographer.options.args["Vendors"])

    if (not self.db.faction.notes) then self.db.faction.notes = {} end
end

function mod:OnEnable()
	if Cartographer_Notes then
		if not self.iconsregistered then
			for k,v in pairs(self.icon) do
				Cartographer_Notes:RegisterIcon(k, v)
			end
			self.iconsregistered = true
		end

		Cartographer_Notes:RegisterNotesDatabase('Vendors', self.db.faction.notes, self)
	else
		Cartographer:ToggleModuleActive(self, false)
	end

    self:RegisterEvent("MERCHANT_SHOW", function() self:RegisterVendor(UnitName("NPC")) end, 2)
end

function mod:OnDisable()
	self:UnregisterAllEvents()

	if Cartographer_Notes then
		Cartographer_Notes:UnregisterNotesDatabase('Vendors')
	end
end

function mod:SetNote(vendor)
	local x, y = GetPlayerMapPosition("player")
	if x == 0 and y == 0 then return end
	local zone = GetRealZoneText()
    
    local note = {}
    note.icon = L["Vendor"]
    note.title = vendor.name
    note.info = L["Vendor"]
    
	for zone, x_p, y_p, icon, creator, data in Cartographer_Notes:IterateNearbyNotes(zone, x, y, 5, "Vendors", nil, true) do
		if data.title == note.title then
			Cartographer_Notes:DeleteNote(zone, x_p, y_p)
		end
	end
	Cartographer_Notes:SetNote(zone, x, y, note.icon, "Vendors", note)
end

function mod:RegisterVendor(vendor)
    if (not vendor) then return end
    if (type(vendor) ~= "string") then return end
    
    if (not mod.vendors) then mod.vendors = {} end
    if (not mod.vendors[vendor]) then 
        mod.vendors[vendor] = {} 
        mod.vendors[vendor].name = vendor
    end
    
    self:SetNote(mod.vendors[vendor])
end

function mod:IsNoteHidden(zone, id, vendortype)
	return not self.db.profile.filter[vendortype]
end

function mod:IsShowingVendors(vendortype)
	return self.db.profile.filter[vendortype]
end

function mod:ToggleShowingVendors(vendortype, value)
	if value == nil then
		value = not self.db.profile.filter[vendortype]
	end
	self.db.profile.filter[vendortype] = value

	Cartographer_Notes:RefreshMap()
end
